@@ -34,8 +34,20 @@
 #ifndef __BGEOT_CONFIG_H
 #define __BGEOT_CONFIG_H
 
+#include <getfem_arch_config.h>
+
 #include <dal_except.h>
 
+#ifdef GETFEM_HAVE_QDLIB
+#  define NO_INLINE
+#  ifdef GETFEM_QDLIB_USE_QUAD
+#    include <qd.h>
+#  else
+#    include <dd.h>
+#  endif
+#  include <x86.h>
+#endif
+
 namespace bgeot
 {
   static const size_t ST_NIL = size_t(-1);
@@ -43,7 +55,23 @@ namespace bgeot
   typedef dal::uint16_type short_type;
   typedef size_t size_type;
   typedef double scalar_type;
-
+#ifndef GETFEM_HAVE_QDLIB
+  typedef double long_scalar_type;
+  typedef double opt_long_scalar_type;
+# define LONG_SCALAR_EPS 1E-16
+# define LONG_SCAL(xx) long_scalar_type(xx)
+#else
+#  ifdef GETFEM_QDLIB_USE_QUAD
+  typedef qd_real long_scalar_type;
+  typedef qd_real opt_long_scalar_type;
+# define LONG_SCALAR_EPS 1E-64
+#  else
+  typedef dd_real long_scalar_type;
+  typedef dd_real opt_long_scalar_type;
+# define LONG_SCALAR_EPS 1E-32
+#  endif
+#  define LONG_SCAL(xx) long_scalar_type(#xx) /* string assignment to preserve the precision */
+#endif
   using dal::dimension_error;
   using dal::file_not_found_error;
   using dal::internal_error;
