@@ -139,6 +139,7 @@ namespace bgeot
     
     mesh_convex_ind_ct(const mesh_structure &ms, size_type ip)
       { _begin = iterator(ms, ip); }
+    mesh_convex_ind_ct(void) { }
     
     const iterator &begin() const { return _begin; }
     const iterator &end() const { return _end; }
@@ -420,10 +421,12 @@ namespace bgeot
   typedef dal::dynamic_tree_sorted<edge_list_elt> edge_list;
   
 
-  void mesh_edge_list_convex(const mesh_structure &m, size_type i, 
-			     edge_list &el, bool merge_convex = true);
-  void mesh_edges_list(const mesh_structure &m, edge_list &el, 
-		       bool merge_convex = true);
+  void mesh_edge_list_convex(pconvex_structure cvs, 
+                             std::vector<size_type> points_of_convex, 
+                             size_type cv_id, edge_list &el, 
+                             bool merge_convex);
+  void mesh_edge_list(const mesh_structure &m, edge_list &el, 
+                      bool merge_convex = true);
 
 
 
