@@ -204,6 +204,7 @@ namespace bgeot
      * of variables. A Horner scheme is used.
      */
     template <class ITER> T eval(const ITER &it) const;
+
     /// Constructor.
     polynomial(void) : std::vector<T>(1)
       { n = 0; d = 0; (*this)[0] = 0.0; }
@@ -299,7 +300,11 @@ namespace bgeot
 	    for ( ; mita != mite; ++mita, ++mitq, ++mit)
 	      *mit = (*mita) + (*mitq); /* on pourrait calculer
 					   directement l'index global. */
+	    //	     cerr << "*= : " << *this << ", itq*ita=" << (*itq) * (*ita) << endl;
+	    //	     cerr << " itq = " << *itq << endl;
+	    //	     cerr << " ita = " << *ita << endl;
 	    add_monomial((*itq) * (*ita), mitot);
+	   
 	  }
       }
     return *this;
@@ -373,13 +378,24 @@ namespace bgeot
     }
   }
 
-
   template<class T> template<class ITER>
     T polynomial<T>::eval(const ITER &it) const {
     power_index mi(dim());
     return horner(mi, dim(), 0, it);
   }
 
+  template<class ITER>
+    typename std::iterator_traits<ITER>::value_type
+        eval_monomial(const power_index &mi, ITER it) {
+    typename std::iterator_traits<ITER>::value_type res
+      = typename std::iterator_traits<ITER>::value_type(1);
+    power_index::const_iterator mit = mi.begin(), mite = mi.end();
+    for ( ; mit != mite; ++mit, ++it)
+      for (short_type l = 0; l < *mit; ++l)
+	res *= *it;
+    return res;
+  }
+
 
   /// Print P to the output stream o. for instance cout $<<$ P;
   template<class T>  std::ostream &operator <<(std::ostream &o,
@@ -408,7 +424,7 @@ namespace bgeot
     return o;
   }
   
-  typedef polynomial<scalar_type> base_poly;
+  typedef polynomial<opt_long_scalar_type> base_poly;
 
   /* usual constant polynomials  */
   
