@@ -1,4 +1,4 @@
-/* -*- c++ -*- (enables emacs c++ mode)                                    */
+/* -*- c++ -*- (enables emacs c++ mode)     */
 /* 
    "sparse" tensors: these are not handle like sparse matrices
    
@@ -6,7 +6,8 @@
    As an example, let say that we have a tensor t(i,j,k,l) of
    dimensions 4x2x3x3, with t(i,j,k,l!=k) == 0. 
    
-   Then the tensor shape will be represented by a set of 3 objects of type 'tensor_mask':
+   Then the tensor shape will be represented by a set of 3 objects of type
+     'tensor_mask':
    mask1: {i}, "1111"
    mask2: {j}, "11"
    mask3: {k,l}, "100"
@@ -14,14 +15,16 @@
                  "001"
    They contain a binary tensor indicating the non-null elements.
    
-   The set of these three masks define the shape of the tensor (class tensor_shape)
+   The set of these three masks define the shape of the tensor
+   (class tensor_shape)
 
-   If we add information about the location of the non-null elements (by mean of strides),
-   then we have an object of type 'tensor_ref'
+   If we add information about the location of the non-null elements
+   (by mean of strides), then we have an object of type 'tensor_ref'
 
    
-   Iteration on the data of one or more tensor should be done via the 'multi_tensor_iterator',
-   which can iterate over common non-null elements of a set of tensors.
+   Iteration on the data of one or more tensor should be done via the
+   'multi_tensor_iterator', which can iterate over common non-null
+   elements of a set of tensors.
 
 
    maximum (virtual) number of elements in a tensor : 2^31
@@ -29,6 +32,8 @@
 
    "ought to be enough for anybody"
 */
+#ifndef BGEOT_SPARSE_TENSORS
+#define BGEOT_SPARSE_TENSORS
 
 #include <bgeot_config.h>
 #include <dal_except.h>
@@ -293,17 +298,17 @@ namespace bgeot {
 
     static void find_linked_masks(dim_type mnum, const tensor_shape &ts1, const tensor_shape &ts2, 
 				dal::bit_vector& treated1, dal::bit_vector& treated2, 
-				std::vector<const tensor_mask*>& lst1,
-				std::vector<const tensor_mask*>& lst2) {
+				std::vector<const tensor_mask*>& lstA,
+				std::vector<const tensor_mask*>& lstB) {
       // gare aux boucles infinies si aucun des indices n'est valide
       assert(mnum < ts1.masks().size());
       assert(!treated1[mnum]);
       treated1.add(mnum);
-      lst1.push_back(&ts1.mask(mnum));
+      lstA.push_back(&ts1.mask(mnum));
       for (dim_type i=0; i < ts1.mask(mnum).indexes().size(); ++i) {
 	dim_type ii = ts1.mask(mnum).indexes()[i];
 	if (ts2.index_is_valid(ii) && !treated2[ts2.index_to_mask_num(ii)])
-	  find_linked_masks(ts2.index_to_mask_num(ii),ts2,ts1,treated2,treated1,lst2,lst1);
+	  find_linked_masks(ts2.index_to_mask_num(ii),ts2,ts1,treated2,treated1,lstB,lstA);
       }
     }
 
@@ -420,22 +425,22 @@ namespace bgeot {
 	  if (ts2.dim(i) != dim(i)) DAL_INTERNAL_ERROR("");
 
       tensor_mask_container new_mask;
-      dal::bit_vector mask_treated1; if (masks().size()) mask_treated1.sup(0,masks().size()-1);
-      dal::bit_vector mask_treated2; if (ts2.masks().size()) mask_treated2.sup(0,ts2.masks().size()-1);
-      std::vector<const tensor_mask*> lst1, lst2; lst1.reserve(10); lst2.reserve(10);
+      dal::bit_vector mask_treated1; mask_treated1.sup(0,masks().size());
+      dal::bit_vector mask_treated2; mask_treated2.sup(0,ts2.masks().size());
+      std::vector<const tensor_mask*> lstA, lstB; lstA.reserve(10); lstB.reserve(10);
       for (index_type i = 0; i < ndim(); ++i) {
 	dim_type i1 = index_to_mask_num(i);
 	dim_type i2 = ts2.index_to_mask_num(i);
-	lst1.clear(); lst2.clear();
+	lstA.clear(); lstB.clear();
 	if (index_is_valid(i) && !mask_treated1[i1])
 	  find_linked_masks(i1, *this, ts2, mask_treated1, mask_treated2,
-			    lst1, lst2);
+			    lstA, lstB);
 	else if (ts2.index_is_valid(i) && !mask_treated2[i2])
 	  find_linked_masks(i2, ts2, *this, mask_treated2, mask_treated1,
-			    lst2, lst1);
+			    lstB, lstA);
 	else continue;
-	if (!(lst1.size() || lst2.size())) DAL_INTERNAL_ERROR("");
-	new_mask.push_back(tensor_mask(lst1,lst2,and_op));
+	if (!(lstA.size() || lstB.size())) DAL_INTERNAL_ERROR("");
+	new_mask.push_back(tensor_mask(lstA,lstB,and_op));
       }
       _masks = new_mask;
       update_idx2mask();
@@ -866,3 +871,5 @@ namespace bgeot {
     }
   };
 } /* namespace bgeot */
+
+#endif
