@@ -39,6 +39,9 @@
 #include <getfem_assembling_tensors.h>
 namespace getfem
 {
+  /**
+     compute $\|U\|_2$
+  */
   template<class VEC>
   scalar_type asm_L2_norm(const mesh_fem &mf, const VEC &U) {
     return asm_L2_norm(mf,U,mf.convex_index());
@@ -59,6 +62,9 @@ namespace getfem
     return sqrt(v[0]);
   }
 
+  /**
+     compute $\|\nabla U\|_2$
+   */
   template<class VEC>
   scalar_type asm_H1_semi_norm(const mesh_fem &mf, const VEC &U) {
     return asm_H1_semi_norm(mf,U,mf.convex_index());
@@ -79,6 +85,9 @@ namespace getfem
     return sqrt(v[0]);
   }
 
+  /** 
+      compute the H1 norm of U.
+  */
   template<class VEC>
   scalar_type asm_H1_norm(const mesh_fem &mf, const VEC &U) {
     return asm_H1_norm(mf,U,mf.convex_index());
@@ -90,20 +99,32 @@ namespace getfem
   }
   
 
-  /* generic mass matrix assembly (on the whole mesh or on the specified boundary) */
+  /** 
+      generic mass matrix assembly (on the whole mesh or on the specified boundary) 
+  */
   template<class MAT>
   void asm_mass_matrix(MAT &M, const mesh_fem &mf_u1, size_type boundary=size_type(-1)) {
     asm_mass_matrix(M,mf_u1,mf_u1, boundary);
   }
+
+  /** 
+      generic mass matrix assembly (on the whole mesh or on the specified boundary) 
+  */
   template<class MAT>
   void asm_mass_matrix(MAT &M, const mesh_fem &mf_u1, const mesh_fem &mf_u2,
 		       size_type boundary=size_type(-1))
   {
     generic_assembly assem;
-    if (mf_u1.get_qdim() == 1 && mf_u2.get_qdim() == 1)
-      assem.set("M(#1,#2)+=comp(Base(#1).Base(#2))");
+    if (&mf_u1 != &mf_u2)
+      if (mf_u1.get_qdim() == 1 && mf_u2.get_qdim() == 1)
+	assem.set("M(#1,#2)+=comp(Base(#1).Base(#2))");
+      else
+	assem.set("M(#1,#2)+=comp(vBase(#1).vBase(#2))(:,i,:,i);");
     else
-      assem.set("M(#1,#2)+=comp(vBase(#1).vBase(#2))(:,i,:,i);");
+      if (mf_u1.get_qdim() == 1 && mf_u2.get_qdim() == 1)
+	assem.set("M(#1,#2)+=sym(comp(Base(#1).Base(#2)))");
+      else
+	assem.set("M(#1,#2)+=sym(comp(vBase(#1).vBase(#2))(:,i,:,i));");
     assem.push_mf(mf_u1);
     assem.push_mf(mf_u2);
     assem.push_mat(M);
@@ -113,7 +134,7 @@ namespace getfem
   }
 
 
-  /* 
+  /**
      source term (for both volumic sources and boundary (neumann) sources  
   */
   template<class VECT1, class VECT2>
@@ -135,7 +156,7 @@ namespace getfem
   }
 
 
-  /*
+  /**
     assembles $\int{qu.v}$
 
     (if $u$ is a vector field of size $N$, $q$ is a square matrix $N\timesN$
@@ -176,7 +197,7 @@ namespace getfem
   }
 
 
-  /* 
+  /** 
      Stiffness matrix for linear elasticity, with Lam coefficients
   */
   template<class MAT, class VECT>
@@ -186,7 +207,7 @@ namespace getfem
 						   VECT &LAMBDA, VECT &MU)
   {
     /* e = strain tensor,
-       M = 2*mu*e(u):e(v) + mu*tr(e(u))*tr(e(v))
+       M = 2*mu*e(u):e(v) + lambda*tr(e(u))*tr(e(v))
     */
     generic_assembly assem("lambda=data$1(#2); mu=data$2(#2);"
 			   "t=comp(vGrad(#1).vGrad(#1).Base(#2));"
@@ -201,7 +222,7 @@ namespace getfem
     assem.volumic_assembly();
   }
 
-  /* 
+  /** 
      Stiffness matrix for linear elasticity, with a general Hooke  tensor 
   */
   template<class MAT, class VECT>
@@ -224,9 +245,10 @@ namespace getfem
     assem.volumic_assembly();
   }
 
-  /* two-in-one assembly of stokes equation:
-     linear elasticty part and p.div(v) term are assembled at the
-     same time. */
+  /** two-in-one assembly of stokes equation:
+      linear elasticty part and p.div(v) term are assembled at the
+      same time. 
+  */
   template<class MAT, class VECT>
     void asm_stokes(MAT &K, MAT &B, 
 		    const mesh_fem &mf_u,
@@ -247,8 +269,11 @@ namespace getfem
     assem.volumic_assembly();
   }
 
+  /**
+     assembly of $\int_\Omega a(x)\nabla u.\nabla v$ , where $a(x)$ is scalar.
+  */
   template<class MAT, class VECT>
-    void asm_stiffness_matrix_for_laplacian(MAT &RM, const mesh_fem &mf,
+    void asm_stiffness_matrix_for_laplacian(MAT &M, const mesh_fem &mf,
 					   const mesh_fem &mfdata, VECT &A)
   {
     generic_assembly assem("a=data$1(#2); M$1(#1,#1)+=sym(comp(Grad(#1).Grad(#1).Base(#2))(:,i,:,i,j).a(j))");
@@ -256,10 +281,27 @@ namespace getfem
     assem.push_mf(mf);
     assem.push_mf(mfdata);
     assem.push_data(A);
-    assem.push_mat(RM);
+    assem.push_mat(M);
     assem.volumic_assembly();
   }
 
+  /**
+     assembly of $\int_\Omega A(x)\nabla u.\nabla v$, where $A(x)$ is a matrix.
+  */
+  template<class MAT, class VECT>
+    void asm_stiffness_matrix_for_scalar_elliptic(MAT &M, const mesh_fem &mf,
+						  const mesh_fem &mfdata,
+						  VECT &A)
+  {
+    generic_assembly assem("a=data$1(mdim(#1),mdim(#1),#2); M$1(#1,#1)+=comp(Grad(#1).Grad(#1).Base(#2))(:,i,:,j,k).a(j,i,k)");
+    assem.push_mf(mf);
+    assem.push_mf(mfdata);
+    assem.push_data(A);
+    assem.push_mat(M);
+    assem.volumic_assembly();
+  }
+
+
   /* ********************************************************************* */
   /*                                                                       */
   /*  Algorithmes d'assemblage pour quelques problemes elliptiques.        */
@@ -534,7 +576,7 @@ namespace getfem
 	  pmec->gen_compute_on_face(t,mf_u.linked_mesh().points_of_convex(cv),
 				    f);
 	  base_tensor::iterator p = t.begin();
-	  scalar_type vmax = bgeot::vect_norminf(t);
+	  scalar_type vmax = gmm::vect_norminf(base_vector(t));
 
 	  for (size_type j = 0; j < nbdof_u; j++) {
 	    size_type dof_j = mf_u.ind_dof_of_element(cv)[j];
@@ -598,7 +640,8 @@ namespace getfem
     dal::bit_vector nn = mf_u.convex_index(), nf;
     pfem pf_u, pf_rh;
     
-    if (mf_rh.get_qdim() != 1) DAL_THROW(std::invalid_argument, "invalid data mesh fem for dirichlet");
+    if (mf_rh.get_qdim() != 1)
+      DAL_THROW(std::invalid_argument, "invalid data mesh fem for dirichlet");
     asm_qu_term(M, mf_u, mf_rh, H, boundary);
     asm_source_term(B, mf_u, mf_rh, R, boundary);
 
@@ -663,20 +706,23 @@ namespace getfem
       }
     }
   }
+
   template<class MAT, class VECT>
   void asm_dirichlet_constraints(MAT &M, VECT &B, const mesh_fem &mf_u,
 				 const mesh_fem &mf_rh,
 				 const VECT &R, size_type boundary) {
-    if (mf_rh.get_qdim() != 1) DAL_THROW(std::invalid_argument, "mf_rh should be a scalar (qdim=1) mesh_fem");
+    if (mf_rh.get_qdim() != 1) 
+      DAL_THROW(std::invalid_argument,
+		"mf_rh should be a scalar (qdim=1) mesh_fem");
     size_type N = mf_rh.nb_dof(), Q=mf_u.get_qdim();
     VECT H(dal::sqr(mf_u.get_qdim())*N); gmm::clear(H);
     
     for (size_type i=0; i < N; ++i) {
       for (size_type q=0; q < Q; ++q) {
-	H(i*Q*Q+q*Q+q)=1;
+	H[i*Q*Q+q*Q+q]=1;
       }
     }
-    asm_dirichlet_constraints(M,B,mf_u,mf_rh,H,R,boundary);
+    asm_dirichlet_constraints(M, B, mf_u, mf_rh, H, R, boundary);
   }
 
   /* old version, pre-Qdim */
@@ -742,7 +788,8 @@ namespace getfem
   /* ********************************************************************* */
 
   template<class MATRM, class MESH_FEM>
-    void mass_matrix(MATRM &M, const MESH_FEM &mf1, const MESH_FEM &mf2, dim_type N)
+    void mass_matrix(MATRM &M, const MESH_FEM &mf1,
+		     const MESH_FEM &mf2, dim_type N)
   {
     DAL_WARNING(3, "obsolete function - use asm_mass_matrix");
     size_type cv, nbd1, nbd2;
@@ -912,8 +959,7 @@ namespace getfem
 					const mesh_fem &mf,
 					size_type boundary,
 					const VECT2 &F)
-  { /* Y-a-il un moyen plus performant ? */
-    // Marche uniquement pour des ddl de lagrange.
+  { // Marche uniquement pour des ddl de lagrange.
     size_type cv, Q=mf.get_qdim();
     dal::bit_vector nn = mf.convex_index();
     dal::bit_vector nndof = mf.dof_on_boundary(boundary);
@@ -1050,16 +1096,19 @@ namespace getfem
     dal::dynamic_array<TEMP_VECT> base_img_inv;
     size_type nb_bimg = 0;
     
+    if (!(gmm::is_col_matrix(D)))
+      DAL_WARNING(3,
+		  "Dirichlet_nullspace is inefficient when D is a row matrix");
     // First, detection of null columns of D, and already orthogonals 
     // vectors of the image of D.
     dal::bit_vector nn;
     for (size_type i = 0; i < nbd; ++i) {
-      gmm::clear(e); e[i] = 1.0; gmm::copy(e, f);
+      gmm::clear(e); e[i] = 1.0;
       gmm::mult(D, e, aux);
-      if (gmm::vect_norm2(aux) < 1.0E-8) { // scaler sur l'ensemble de D ...
-	//	cerr << "nbase=" << nbase << " i =" << i;
+      scalar_type n = gmm::vect_norm2(aux);
+
+      if (n < 1.0E-8) { // scaler sur l'ensemble de D ...
 	G(i, nbase++) = 1.0; nn[i] = true;
-	//	cerr << "-> G=" << G(i,nbase-1) << "\n";
       }
       else {
 	bool good = true;
@@ -1067,7 +1116,7 @@ namespace getfem
 	  if (dal::abs(gmm::vect_sp(aux, base_img[j])) > 1.0E-16)
 	    { good = false; break; }
 	if (good) {
-	  scalar_type n = gmm::vect_norm2(aux);
+	  gmm::copy(e, f);
 	  gmm::scale(f, 1.0 / n); gmm::scale(aux, 1.0 / n);
 	  base_img_inv[nb_bimg] = TEMP_VECT(nbd);
 	  gmm::copy(f, base_img_inv[nb_bimg]);
@@ -1111,7 +1160,6 @@ namespace getfem
       scalar_type c = gmm::vect_sp(base_img[i], UD);
       gmm::add(gmm::scaled(base_img_inv[i], c), UDD);
     }
-
     // Orthogonalisation of the basis of the kernel of D.
     for (size_type i = nb_triv_base + 1; i < nbase; ++i) {
       for (size_type j = nb_triv_base; j < i; ++j) {
@@ -1129,7 +1177,8 @@ namespace getfem
     // Test ...
     gmm::mult(D, UDD, gmm::scaled(UD, -1.0), aux);
     if (gmm::vect_norm2(aux) > 1.0E-12)
-      cerr << "Dirichlet condition not well inverted: residu=" << gmm::vect_norm2(aux) << "\n";
+      DAL_WARNING(2, "Dirichlet condition not well inverted: residu="
+		  << gmm::vect_norm2(aux));
 
     return nbase;
   }
