@@ -75,7 +75,7 @@ namespace getfem
 
       o << "DIM = " << int(pgt->dim()) << endl;
 
-      if (!(pf1->is_equivalent())) 
+      if (pf1->need_G()) 
 	transfert_to_G(G, mf.linked_mesh().points_of_convex(cv));
 
       if (pf1->target_dim() != 1)
@@ -251,7 +251,8 @@ namespace getfem
   /* ********************************************************************* */
 
   template<class VECT>
-    void interpolation_solution_same_mesh(mesh_fem &mf_source, mesh_fem &mf_target,
+    void interpolation_solution_same_mesh(mesh_fem &mf_source,
+					  mesh_fem &mf_target,
 					  const VECT &U, VECT &V)
   {
     dal::bit_vector nn = mf_source.convex_index();
@@ -276,7 +277,7 @@ namespace getfem
       size_type nbd_t = pf_t->nb_dof();
       coeff.resize(nbd_s);
 
-      if (!(pf_s->is_equivalent())) 
+      if (pf_s->need_G()) 
 	transfert_to_G(G, mf_source.linked_mesh().points_of_convex(cv));
 
       if (pf_s->target_dim() != 1 || pf_t->target_dim() != 1)
@@ -311,8 +312,7 @@ namespace getfem
 
   template<class VECT>
     void interpolation_solution(mesh_fem &mf_source, mesh_fem &mf_target,
-				const VECT &U, VECT &V)
-  {
+				const VECT &U, VECT &V) {
     size_type cv;
     base_node val(1);
     bgeot::geotrans_inv gti;
@@ -363,18 +363,15 @@ namespace getfem
     // il faudrait controler que tous les ddl de mf_target sont de
     // type lagrange
     dal::bit_vector nn = mf_source.convex_index(), ddl_touched;
-    ddl_touched.add(0, mf_target.nb_dof()-1);
+    ddl_touched.add(0, mf_target.nb_dof());
 
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
-      bgeot::pgeometric_trans pgt = mf_source.linked_mesh().trans_of_convex(cv);
-      //cout << "dealing with convex " << cv << " remaining " << nn.card() << endl;
+      bgeot::pgeometric_trans pgt=mf_source.linked_mesh().trans_of_convex(cv);
       size_type nb = gti.points_in_convex(mf_source.linked_mesh().convex(cv),
 					  pgt, ptab, itab);
-      //cout << "nb points in this convex " << nb << endl;
-      // cout << "convex : " << mf_source.linked_mesh().convex(cv) << endl;
       pfem pf_s = mf_source.fem_of_element(cv);
-      if (!(pf_s->is_equivalent())) 
+      if (pf_s->need_G()) 
 	transfert_to_G(G, mf_source.linked_mesh().points_of_convex(cv));
       size_type nbd_s = pf_s->nb_dof();
       coeff.resize(nbd_s);
@@ -383,7 +380,6 @@ namespace getfem
 	size_type dof_t = gti_pt_2_target_dof[itab[i]].first;
 	size_type nrep  = gti_pt_2_target_dof[itab[i]].second;
 	assert(nrep>0);
-	// cout << "dealing with ddl : " << itab[i] << "  coords : " << mf_target.point_of_dof(itab[i]) << " internal coords : " << ptab[i] << endl;
 	if (ddl_touched[dof_t])
 	{ // inverser les deux boucles pour gagner du temps ?
 	  // Il faut verifier que le ddl est bien de Lagrange ...
@@ -391,7 +387,6 @@ namespace getfem
 	    for (size_type j = 0; j < nbd_s; ++j) {
 	      size_type dof_s = mf_source.ind_dof_of_element(cv)[j*nrep+k];
 	      coeff[j] = U[dof_s];
-	      //cerr << "dof_s=" << dof_s << " dof_t=" << dof_t << " k=" << k << " i=" << i << endl;
 	    }
 	    pf_s->interpolation(ptab[i], G, pgt, coeff, val);
 	    V[dof_t + k] = val[0];
@@ -406,6 +401,67 @@ namespace getfem
 	   << " all points have not been touched" << endl;
   }
 
+  /* ********************************************************************* */
+  /*                                                                       */
+  /*  interpolation of a solution on a set of points.                      */
+  /*                                                                       */
+  /* ********************************************************************* */
+
+  template<class VECT>
+    void interpolation_solution(mesh_fem &mf_source, bgeot::geotrans_inv &gti,
+				const VECT &U, VECT &V) {
+    size_type cv;
+    base_node val(1);
+    dal::dynamic_array<base_node> ptab;
+    dal::dynamic_array<size_type> itab;
+    base_vector coeff;
+    base_matrix G;
+
+    size_type qdim = mf_source.get_qdim();
+
+    dal::bit_vector nn = mf_source.convex_index(), ddl_touched;
+    ddl_touched.add(0, gti.nb_points());
+
+    for (cv << nn; cv != ST_NIL; cv << nn)
+    {
+      bgeot::pgeometric_trans pgt=mf_source.linked_mesh().trans_of_convex(cv);
+      size_type nb = gti.points_in_convex(mf_source.linked_mesh().convex(cv),
+					  pgt, ptab, itab);
+      pfem pf_s = mf_source.fem_of_element(cv);
+
+      if (pf_s->need_G())
+	transfert_to_G(G, mf_source.linked_mesh().points_of_convex(cv));
+      // cerr << "is_equiv:" << pf_s->is_equivalent() << ",inerp: G=" << G << ",nrow=" << G.nrows() << ", ncols=" << G.ncols() << endl;
+      size_type nbd_s = pf_s->nb_dof();
+      coeff.resize(nbd_s);
+      for (size_type i = 0; i < nb; ++i)
+      {
+	size_type dof_t = itab[i];
+	size_type nrep  = qdim / pf_s->target_dim();
+	if (ddl_touched[dof_t])
+	{ // inverser les deux boucles pour gagner du temps ?
+	  // Il faut verifier que le ddl est bien de Lagrange ...
+	  for (size_type k = 0; k < nrep; ++k) {
+	    for (size_type j = 0; j < nbd_s; ++j) {
+	      size_type dof_s = mf_source.ind_dof_of_element(cv)[j*nrep+k];
+	      coeff[j] = U[dof_s];
+	    }
+	    // cerr << "cv=" << cv << ", ptab[" << i << "]=" << ptab[i] << ", coeff=" << coeff << endl;
+	    pf_s->interpolation(ptab[i], G, pgt, coeff, val);
+	    V[dof_t*qdim + k] = val[0];
+	    ddl_touched.sup(dof_t);
+	  }
+	}
+      }
+    }
+    if (ddl_touched.card() != 0)
+      cerr << "WARNING : in interpolation_solution,"
+	   << " all points have not been touched" << endl;
+  }
+
+
+
+
 
 
   /*
@@ -433,7 +489,7 @@ namespace getfem
       size_type nbd2 = pfe->nb_dof();
       coeff.resize(nbd1);
 
-      if (!(pf1->is_equivalent())) 
+      if (pf1->need_G()) 
 	transfert_to_G(G, mf.linked_mesh().points_of_convex(cv));
 
       if (pf1->target_dim() != 1)
@@ -492,24 +548,20 @@ namespace getfem
     // type lagrange
 
     dal::bit_vector nn = mf.convex_index(), ddl_touched;
-    ddl_touched.add(0, nb-1);
+    ddl_touched.add(0, nb);
 
     for (cv << nn; cv != ST_NIL; cv << nn)
     {
       bgeot::pgeometric_trans pgt = mf.linked_mesh().trans_of_convex(cv);
-      //cout << "dealing with convex " << cv << " remaining " << nn.card() << endl;
       nb = gti.points_in_convex(mf.linked_mesh().convex(cv),
-			    mf.linked_mesh().trans_of_convex(cv), ptab, itab);
-      //cout << "nb points in this convex " << nb << endl;
-      // cout << "convex : " << mf.linked_mesh().convex(cv) << endl;
+				pgt, ptab, itab);
       pfem pfe = mf.fem_of_element(cv);
-      if (!(pfe->is_equivalent())) 
+      if (pfe->need_G()) 
 	transfert_to_G(G, mf.linked_mesh().points_of_convex(cv));
       size_type nbd1 = pfe->nb_dof();
       coeff.resize(nbd1);
       for (size_type i = 0; i < nb; ++i)
       {
-	// cout << "dealing with ddl : " << itab[i] << "  coords : " << mf_target.point_of_dof(itab[i]) << " internal coords : " << ptab[i] << endl;
 	if (ddl_touched[itab[i]])
 	{ // inverser les deux boucles pour gagner du temps ?
 	  // Il faut verifier que le ddl est bien de Lagrange ...
@@ -522,8 +574,6 @@ namespace getfem
 	    pfe->interpolation(ptab[i], G, pgt, coeff, val);
 	    pt3[k] = val[0];
 	  }
-	  
-	  // cout << "Rsultat : " << pt3 << endl;
 	  for (size_type j = 0; j < P; ++j) V[itab[i]*P+j] = pt3[j];
 	  ddl_touched.sup(itab[i]);
 	}
