@@ -50,8 +50,8 @@ namespace getfem
 
       bgeot::pconvex_structure cvs;
 
-      mutable std::vector<scalar_type> int_monomials;
-      mutable std::vector< std::vector<scalar_type> > int_face_monomials;
+      mutable std::vector<long_scalar_type> int_monomials;
+      mutable std::vector< std::vector<long_scalar_type> > int_face_monomials;
 
     public :
 
@@ -60,17 +60,17 @@ namespace getfem
       /// {Structure of convex of reference.  
       bgeot::pconvex_structure structure(void) const { return cvs; }
 
-      virtual scalar_type int_monomial(const bgeot::power_index &power)
+      virtual long_scalar_type int_monomial(const bgeot::power_index &power)
 	const = 0;
-      virtual scalar_type int_monomial_on_face(const bgeot::power_index
+      virtual long_scalar_type int_monomial_on_face(const bgeot::power_index
 					       &power, short_type f) const = 0;
 
       /// Evaluate the integral of the polynomial P on the reference element.
-      scalar_type int_poly(const base_poly &P) const;
+      long_scalar_type int_poly(const base_poly &P) const;
       /** Evaluate the integral of the polynomial P on the face f of the
        *    reference element.
        */
-      scalar_type int_poly_on_face(const base_poly &P, short_type f) const;
+      long_scalar_type int_poly_on_face(const base_poly &P, short_type f) const;
 
       virtual ~poly_integration() {}
 
@@ -100,7 +100,8 @@ namespace getfem
       std::vector<scalar_type> int_coeffs;
       std::vector<size_type> repartition;
 
-      std::vector<PT_TAB> pt_to_store;
+    // index 0 : points for volumic integration, index > 0 : points for faces
+      std::vector<PT_TAB> pt_to_store; 
       bool valid;
 
     public :
@@ -140,7 +141,11 @@ namespace getfem
       scalar_type coeff_on_face(short_type f, size_type i) const
       { return int_coeffs[repartition[f] + i]; }
 
-      void add_point(base_node pt, scalar_type w, short_type f=short_type(-1));
+      void add_point(const base_node &pt, scalar_type w,
+		     short_type f=short_type(-1));
+      void add_point_norepeat(const base_node &pt, scalar_type w,
+			      short_type f=short_type(-1));
+      void add_point_full_symmetric(base_node pt, scalar_type w);
       void add_method_on_face(pintegration_method ppi, short_type f);
       void valid_method(void);
 
@@ -233,7 +238,11 @@ namespace getfem
   pintegration_method exact_classical_im(bgeot::pgeometric_trans pgt);
   
   std::string name_of_int_method(pintegration_method p);
-
+  class mesh_precomposite;
+  class mesh_fem;
+  papprox_integration composite_approx_int_method(const mesh_precomposite &mp, 
+						  const mesh_fem &mf,
+						  bgeot::pconvex_ref cr);
 }  /* end of namespace getfem.                                            */
 
 
