@@ -32,14 +32,6 @@
 #ifndef __GMM_INTERFACE_H
 #define __GMM_INTERFACE_H
 
-namespace bgeot {
-  template <class T, int N> class fsvector;
-  template <class T> class vsvector;
-  template <class VECT> class PT;
-  template <class T, int N> class fsmatrix;
-  template <class T> class vsmatrix;
-}
-
 namespace gmm {
 
   /* ********************************************************************* */
@@ -49,7 +41,7 @@ namespace gmm {
   /*   v[i] allows to access to the ith component of v.                    */
   /*   linalg_traits<Vector> should be filled with appropriate definitions */
   /*                                                                       */
-  /*   for a plain vector : the minimum is two random iterators (begin and */
+  /*   for a dense vector : the minimum is two random iterators (begin and */
   /*                        end) and a const void * pointer to a valid     */
   /*                        origin.                                        */
   /*   for a sparse vector : the minimum is two forward iterators, with    */
@@ -64,15 +56,16 @@ namespace gmm {
   /*   m(i, j) allows to access to the element at row i and column j.      */
   /*   linalg_traits<Matrix> should be filled with appropriate definitions */
   /*                                                                       */
-  /* What is needed for an iterator on plain vector                        */
+  /* What is needed for an iterator on dense vector                        */
   /*    to be standard random access iterator                              */
   /*                                                                       */
   /* What is needed for an iterator on a sparse vector                     */
-  /*    to be a standard forward iterator                                  */
+  /*    to be a standard bidirectional iterator                            */
+  /*    elt should be sorted with increasing indices.                      */
   /*    it.index() gives the index of the non-zero element.                */
   /*                                                                       */
   /* Remark : If original iterators are not convenient, they could be      */
-  /*   redefined and interfaced in linalg_traits<Vector> without changin   */
+  /*   redefined and interfaced in linalg_traits<Vector> without changing  */
   /*   the original Vector type.                                           */
   /*                                                                       */
   /* ********************************************************************* */
@@ -96,7 +89,8 @@ namespace gmm {
     size_type _size;
 
     simple_vector_ref(ref_V v)
-      : _begin(vect_begin(const_cast<V&>(v))), _end(vect_end(const_cast<V&>(v))), 
+      : _begin(vect_begin(const_cast<V&>(v))), 
+	_end(vect_end(const_cast<V&>(v))), 
 	origin(linalg_origin(const_cast<V&>(v))),
 	_size(vect_size(v)) {}
 
@@ -127,7 +121,8 @@ namespace gmm {
     static iterator end(this_type &v) { return v._end; }
     static const_iterator end(const this_type &v) { return v._end; }
     static const void* origin(const this_type &v) { return v.origin; }
-    static void do_clear(this_type &v) { clear_type()(v.origin, v._begin, v._end); }
+    static void do_clear(this_type &v)
+    { clear_type()(v.origin, v._begin, v._end); }
   };
 
   template <class PT>
@@ -153,9 +148,9 @@ namespace gmm {
     typedef T& reference;
     typedef typename this_type::iterator iterator;
     typedef typename this_type::const_iterator const_iterator;
-    typedef abstract_plain storage_type;
-    typedef plain_access<iterator,const_iterator> access_type;
-    typedef plain_clear<iterator> clear_type;
+    typedef abstract_dense storage_type;
+    typedef dense_access<iterator,const_iterator> access_type;
+    typedef dense_clear<iterator> clear_type;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v) { return v.begin(); }
     static const_iterator begin(const this_type &v) { return v.begin(); }
@@ -175,6 +170,10 @@ namespace gmm {
   template <class T> struct linalg_traits<const std::vector<T> > 
     : public linalg_traits<std::vector<T> > {};
 #endif
+
+  template <class T>
+  inline size_type nnz(const std::vector<T>& l) { return l.size(); }
+
   // to be done :  std::valarray<T> ...
 
   /* ********************************************************************* */
@@ -207,9 +206,9 @@ namespace gmm {
     typedef typename std::iterator_traits<IT>::reference reference;
     typedef typename this_type::iterator iterator;
     typedef typename this_type::iterator const_iterator;
-    typedef abstract_plain storage_type;
-    typedef plain_access<iterator,const_iterator> access_type;
-    typedef plain_clear<iterator> clear_type;
+    typedef abstract_dense storage_type;
+    typedef dense_access<iterator,const_iterator> access_type;
+    typedef dense_clear<iterator> clear_type;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v) { return v.begin(); }
     static const_iterator begin(const this_type &v) { return v.begin(); }
@@ -260,9 +259,9 @@ namespace gmm {
     typedef typename std::iterator_traits<IT>::reference reference;
     typedef typename this_type::iterator iterator;
     typedef typename this_type::iterator const_iterator;
-    typedef abstract_plain storage_type;
-    typedef plain_access<iterator,const_iterator> access_type;
-    typedef plain_clear<iterator> clear_type;
+    typedef abstract_dense storage_type;
+    typedef dense_access<iterator,const_iterator> access_type;
+    typedef dense_clear<iterator> clear_type;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v) { return v.begin(); }
     static const_iterator begin(const this_type &v) { return v.begin(); }
@@ -296,13 +295,13 @@ namespace gmm {
     template <class V> tab_ref_index_ref_with_origin(const V &v,
 						     const sub_index &si) :
       dal::tab_ref_index_ref<IT, ITINDEX>(vect_begin(const_cast<V&>(v)),
-					  si.begin(), si.end()),
-                                          origin(linalg_origin(const_cast<V&>(v))) {}
+				   si.begin(), si.end()),
+                                   origin(linalg_origin(const_cast<V&>(v))) {}
     template <class V> tab_ref_index_ref_with_origin(V &v,
 						     const sub_index &si) :
       dal::tab_ref_index_ref<IT, ITINDEX>(vect_begin(const_cast<V&>(v)),
-					  si.begin(), si.end()),
-                                          origin(linalg_origin(const_cast<V&>(v))) {}
+				   si.begin(), si.end()),
+                                   origin(linalg_origin(const_cast<V&>(v))) {}
   };
 
   template <class IT, class ITINDEX>
@@ -315,16 +314,17 @@ namespace gmm {
     typedef typename std::iterator_traits<IT>::reference reference;
     typedef typename this_type::iterator iterator;
     typedef typename this_type::iterator const_iterator;
-    typedef abstract_plain storage_type;
-    typedef plain_access<iterator,const_iterator> access_type;
-    typedef plain_clear<iterator> clear_type;
+    typedef abstract_dense storage_type;
+    typedef dense_access<iterator,const_iterator> access_type;
+    typedef dense_clear<iterator> clear_type;
     static size_type size(const this_type &v) { return v.size(); }
     static iterator begin(this_type &v) { return v.begin(); }
     static const_iterator begin(const this_type &v) { return v.begin(); }
     static iterator end(this_type &v) { return v.end(); }
     static const_iterator end(const this_type &v) { return v.end(); }
     static const void* origin(const this_type &v) { return v.origin; }
-    static void do_clear(this_type &v) { clear_type()(v.origin, v.begin(), v.end()); }
+    static void do_clear(this_type &v)
+    { clear_type()(v.origin, v.begin(), v.end()); }
   };
 
   template <class IT, class ITINDEX> std::ostream &operator <<
@@ -337,96 +337,14 @@ namespace gmm {
     : public  linalg_traits<tab_ref_index_ref_with_origin<IT, ITINDEX> > {};
 #endif
 
-  /* ********************************************************************* */
-  /*		                                         	 	   */
-  /*		Traits for bgeot objects                     		   */
-  /*		                                         		   */
-  /* ********************************************************************* */
-
-  template <class T, int N> struct linalg_traits<bgeot::fsvector<T, N> > {
-    typedef bgeot::fsvector<T, N> this_type;
-    typedef linalg_false is_reference;
-    typedef abstract_vector linalg_type;
-    typedef T value_type;
-    typedef T& reference;
-    typedef typename this_type::iterator iterator;
-    typedef typename this_type::const_iterator const_iterator;
-    typedef abstract_plain storage_type;
-    typedef plain_access<iterator,const_iterator> access_type;
-    typedef plain_clear<iterator> clear_type;
-    static size_type size(const this_type &v) { return v.size(); }
-    static iterator begin(this_type &v) { return v.begin(); }
-    static const_iterator begin(const this_type &v) { return v.begin(); }
-    static iterator end(this_type &v) { return v.end(); }
-    static const_iterator end(const this_type &v) { return v.end(); }
-    static const void* origin(const this_type &v) { return &v; }
-    static void do_clear(this_type &v) { clear_type()(origin(v), begin(v), end(v)); }
-  };
-
-#ifdef USING_BROKEN_GCC295
-  template <class T, int N> struct linalg_traits<const bgeot::fsvector<T, N> >
-    : public linalg_traits<bgeot::fsvector<T, N> > {};
-#endif
-
-  template <class T> struct linalg_traits<bgeot::vsvector<T> > {
-    typedef bgeot::vsvector<T> this_type;
-    typedef linalg_false is_reference;
-    typedef abstract_vector linalg_type;
-    typedef T value_type;
-    typedef T& reference;
-    typedef typename this_type::iterator iterator;
-    typedef typename this_type::const_iterator const_iterator;
-    typedef abstract_plain storage_type;
-    typedef plain_access<iterator,const_iterator> access_type;
-    typedef plain_clear<iterator> clear_type;
-    static size_type size(const this_type &v) { return v.size(); }
-    static iterator begin(this_type &v) { return v.begin(); }
-    static const_iterator begin(const this_type &v) { return v.begin(); }
-    static iterator end(this_type &v) { return v.end(); }
-    static const_iterator end(const this_type &v) { return v.end(); }
-    static const void* origin(const this_type &v) { return &v; }
-    static void do_clear(this_type &v) { clear_type()(origin(v), begin(v), end(v)); }
-  };
-
-#ifdef USING_BROKEN_GCC295
-  template <class T> struct linalg_traits<const bgeot::vsvector<T> > 
-    : public linalg_traits<bgeot::vsvector<T> > {};
-#endif
-
-  template <class VECT> struct linalg_traits<bgeot::PT<VECT> > {
-    typedef bgeot::PT<VECT> this_type;
-    typedef linalg_false is_reference;
-    typedef typename linalg_traits<VECT>::value_type value_type;
-    typedef typename linalg_traits<VECT>::reference reference;
-    typedef abstract_vector linalg_type;
-    typedef typename linalg_traits<VECT>::iterator  iterator;
-    typedef typename linalg_traits<VECT>::const_iterator const_iterator;
-    typedef typename linalg_traits<VECT>::storage_type storage_type;
-    typedef plain_access<iterator,const_iterator> access_type;
-    typedef plain_clear<iterator> clear_type;
-    static size_type size(const this_type &v) { return v.size(); }
-    static iterator begin(this_type &v) { return v.begin(); }
-    static const_iterator begin(const this_type &v) { return v.begin(); }
-    static iterator end(this_type &v) { return v.end(); }
-    static const_iterator end(const this_type &v) { return v.end(); }
-    static const void* origin(const this_type &v) { return &v; }
-    static void do_clear(this_type &v) { clear_type()(origin(v), begin(v), end(v)); }
-  };
-
-#ifdef USING_BROKEN_GCC295
-  template <class VECT> struct linalg_traits<const bgeot::PT<VECT> >
-  : public linalg_traits<bgeot::PT<VECT> > {};
-#endif
-
-  template<class ITER, class MIT> struct plain_compressed_iterator
-  {
+  template<class ITER, class MIT> struct dense_compressed_iterator {
     typedef ITER value_type;
     typedef ITER *pointer;
     typedef ITER &reference;
     typedef ptrdiff_t difference_type;
     typedef std::random_access_iterator_tag iterator_category;
     typedef size_t size_type;
-    typedef plain_compressed_iterator<ITER, MIT> iterator;
+    typedef dense_compressed_iterator<ITER, MIT> iterator;
 
     ITER it;
     size_type N, nrows, ncols;
@@ -452,178 +370,15 @@ namespace gmm {
     bool operator !=(const iterator &i) const { return !(i == *this); }
     bool operator < (const iterator &i) const { return (it < i.it); }
 
-    plain_compressed_iterator(void) {}
-    plain_compressed_iterator(const plain_compressed_iterator<MIT, MIT> &ii)
+    dense_compressed_iterator(void) {}
+    dense_compressed_iterator(const dense_compressed_iterator<MIT, MIT> &ii)
     : it(ii.it), N(ii.N), nrows(ii.nrows),ncols(ii.ncols),origin(ii.origin) {}
-    plain_compressed_iterator(const ITER &iter, size_type n, size_type r,
+    dense_compressed_iterator(const ITER &iter, size_type n, size_type r,
 			      size_type c, const void *o)
       : it(iter), N(n), nrows(r), ncols(c), origin(o) { }
     
   };
 
-  template <class T, int N> struct bgeot_fsmatrix_access {
-    typedef typename linalg_traits<bgeot::fsmatrix<T, N> >::reference
-            reference;
-    typedef typename linalg_traits<bgeot::fsmatrix<T, N> >::value_type
-            value_type;
-    typedef typename linalg_traits<bgeot::fsmatrix<T, N> >::col_iterator
-            iterator;
-    typedef typename linalg_traits<bgeot::fsmatrix<T, N> >::const_col_iterator
-            const_iterator;
-    
-    reference operator()(const iterator &itcol, size_type j)
-    { return (*itcol)[j]; }
-    value_type operator()(const const_iterator &itcol, size_type j)
-    { return (*itcol)[j]; }
-  };
-
-  // row and col iterators could be more specialized to take into account 
-  // the fixed size.
-  template <class T, int N> struct linalg_traits<bgeot::fsmatrix<T, N> > {
-    typedef bgeot::fsmatrix<T, N> this_type;
-    typedef linalg_false is_reference;
-    typedef abstract_matrix linalg_type;
-    typedef T value_type;
-    typedef T& reference;
-    typedef abstract_plain storage_type;
-    typedef tab_ref_reg_spaced_with_origin<typename this_type::iterator>
-            sub_row_type;
-    typedef tab_ref_reg_spaced_with_origin<typename this_type
-            ::const_iterator> const_sub_row_type;
-    typedef plain_compressed_iterator<typename this_type::iterator,
-            typename this_type::iterator> row_iterator;
-    typedef plain_compressed_iterator<typename this_type::const_iterator, 
-            typename this_type::iterator> const_row_iterator;
-    typedef tab_ref_with_origin<typename this_type::iterator> sub_col_type;
-    typedef tab_ref_with_origin<typename this_type::const_iterator>
-            const_sub_col_type;
-    typedef plain_compressed_iterator<typename this_type::iterator,
-            typename this_type::iterator> col_iterator;
-    typedef plain_compressed_iterator<typename this_type::const_iterator, 
-            typename this_type::iterator> const_col_iterator;
-    typedef col_and_row sub_orientation;
-    typedef bgeot_fsmatrix_access<T, N> access_type;
-    static size_type nrows(const this_type &m) { return N; }
-    static size_type ncols(const this_type &m) { return N; } 
-    static const_sub_row_type row(const const_row_iterator &it)
-    { return const_sub_row_type(it.it, it.it + N * N, N, it.origin);  }
-    static const_sub_col_type col(const const_col_iterator &it)
-    { return const_sub_col_type(it.it, it.it + N, it.origin); }
-    static sub_row_type row(const row_iterator &it) 
-    { return sub_row_type(it.it, it.it + N * N, it.nrows, it.origin); }
-    static sub_col_type col(const col_iterator &it)
-    { return sub_col_type(it.it, it.it + N, it.origin); }
-    static row_iterator row_begin(this_type &m)
-    { return row_iterator(m.begin(), 1, N, N, &m); }
-    static row_iterator row_end(this_type &m)
-    { return row_iterator(m.begin() + N, 1, N, N, &m); }
-    static const_row_iterator row_begin(const this_type &m)
-    { return const_row_iterator(m.begin(), 1, N, N, &m); }
-    static const_row_iterator row_end(const this_type &m)
-    { return const_row_iterator(m.begin() + N, 1, N, N, &m); }
-    static col_iterator col_begin(this_type &m)
-    { return col_iterator(m.begin(), N, N, N, &m); }
-    static col_iterator col_end(this_type &m)
-    { return col_iterator(m.end(), N, N, N, &m); }
-    static const_col_iterator col_begin(const this_type &m)
-    { return const_col_iterator(m.begin(), N, N, N, &m); }
-    static const_col_iterator col_end(const this_type &m)
-    { return const_col_iterator(m.end(), N, N, N, &m); }
-    static const void* origin(const this_type &m) { return &m; }
-    static void do_clear(this_type &m);
-  };
-  template <class T, int N>
-  void linalg_traits<bgeot::fsmatrix<T, N> >::do_clear(this_type &m)
-  { std::fill(m.begin(), m.end(), value_type(0)); }
-
-
-#ifdef USING_BROKEN_GCC295
-  template <class T, int N> struct linalg_traits<const bgeot::fsmatrix<T, N> >
-    : public linalg_traits<bgeot::fsmatrix<T, N> > {};
-#endif
-
-  template <class T> struct bgeot_vsmatrix_access {
-    typedef typename linalg_traits<bgeot::vsmatrix<T> >::reference
-            reference;
-    typedef typename linalg_traits<bgeot::vsmatrix<T> >::value_type
-            value_type;
-    typedef typename linalg_traits<bgeot::vsmatrix<T> >::col_iterator
-            iterator;
-    typedef typename linalg_traits<bgeot::vsmatrix<T> >::const_col_iterator
-            const_iterator;
-    
-    reference operator()(const iterator &itcol, size_type j)
-    { return (*itcol)[j]; }
-    value_type operator()(const const_iterator &itcol, size_type j)
-    { return (*itcol)[j]; }
-  };
-
-  template <class T> struct linalg_traits<bgeot::vsmatrix<T> > {
-    typedef bgeot::vsmatrix<T> this_type;
-    typedef linalg_false is_reference;
-    typedef abstract_matrix linalg_type;
-    typedef T value_type;
-    typedef T& reference;
-    typedef abstract_plain storage_type;
-    typedef tab_ref_reg_spaced_with_origin<typename this_type::iterator>
-            sub_row_type;
-    typedef tab_ref_reg_spaced_with_origin<typename this_type
-            ::const_iterator> const_sub_row_type;
-    typedef plain_compressed_iterator<typename this_type::iterator,
-	    typename this_type::iterator> row_iterator;
-    typedef plain_compressed_iterator<typename this_type::const_iterator,
-	    typename this_type::iterator> const_row_iterator;
-    typedef tab_ref_with_origin<typename this_type::iterator> sub_col_type;
-    typedef tab_ref_with_origin<typename this_type::const_iterator>
-            const_sub_col_type;
-    typedef plain_compressed_iterator<typename this_type::iterator,
-	    typename this_type::iterator> col_iterator;
-    typedef plain_compressed_iterator<typename this_type::const_iterator,
-	    typename this_type::iterator> const_col_iterator;
-    typedef col_and_row sub_orientation;
-    typedef bgeot_vsmatrix_access<T> access_type;
-    static size_type nrows(const this_type &m) { return m.nrows(); }
-    static size_type ncols(const this_type &m) { return m.ncols(); }
-    static const_sub_row_type row(const const_row_iterator &it) {
-      return const_sub_row_type(it.it, it.it + it.ncols * it.nrows,
-				it.nrows, it.origin); 
-    }
-    static const_sub_col_type col(const const_col_iterator &it)
-    { return const_sub_col_type(it.it, it.it + it.nrows, it.origin); }
-    static sub_row_type row(const row_iterator &it) {
-      return sub_row_type(it.it, it.it + it.ncols * it.nrows,
-			  it.nrows, it.origin);
-    }
-    static sub_col_type col(const col_iterator &it)
-    { return sub_col_type(it.it, it.it + it.nrows, it.origin); }
-    static row_iterator row_begin(this_type &m)
-    { return row_iterator(m.begin(), 1, m.nrows(), m.ncols(), &m); }
-    static row_iterator row_end(this_type &m)
-    { return row_iterator(m.begin()+m.nrows(), 1, m.nrows(), m.ncols(), &m); }
-    static const_row_iterator row_begin(const this_type &m)
-    { return const_row_iterator(m.begin(), 1, m.nrows(), m.ncols(), &m); }
-    static const_row_iterator row_end(const this_type &m) {
-      return const_row_iterator(m.begin()+m.nrows(), 1, m.nrows(),
-				m.ncols(), &m);
-    }
-    static col_iterator col_begin(this_type &m)
-    { return col_iterator(m.begin(), m.nrows(), m.nrows(), m.ncols(), &m); }
-    static col_iterator col_end(this_type &m)
-    { return col_iterator(m.end(), m.nrows(), m.nrows(), m.ncols(), &m); }
-    static const_col_iterator col_begin(const this_type &m)
-    { return const_col_iterator(m.begin(),m.nrows(),m.nrows(),m.ncols(),&m); }
-    static const_col_iterator col_end(const this_type &m)
-    { return const_col_iterator(m.end(), m.nrows(),m.nrows(),m.ncols(), &m); }
-    static const void* origin(const this_type &m) { return &m; }
-    static void do_clear(this_type &m) { m.fill(value_type(0)); }
-  };
-
-#ifdef USING_BROKEN_GCC295
-  template <class T> struct linalg_traits<const bgeot::vsmatrix<T> >
-    : public linalg_traits<bgeot::vsmatrix<T> > {};
-#endif
-
-
   /* ******************************************************************** */
   /*	    Read only reference on a compressed sparse vector             */
   /* ******************************************************************** */
@@ -673,8 +428,10 @@ namespace gmm {
     const_iterator begin(void) const { return const_iterator(pr, ir); }
     const_iterator end(void) const { return const_iterator(pr+n, ir+n); }
     
-    value_type operator[](size_type i) const
-    { typename linalg_traits<this_type>::access_type()(pr, begin(), end(),i); }
+    value_type operator[](size_type i) const {
+      return typename linalg_traits<this_type>::access_type()(pr, begin(),
+							      end(),i);
+    }
   };
 
   template <class PT1, class PT2, int shift> struct cs_vector_access {
@@ -693,7 +450,7 @@ namespace gmm {
 			  const const_iterator &e, size_type i) {
     if (b.ir == e.ir) return value_type(0);
     PT2 p = std::lower_bound(b.ir, e.ir, i+shift);
-    return (p != b.ir && *p == i+shift) ? b.pr[p-b.ir] : value_type(0);
+    return (*p == i+shift && p != e.ir) ? b.pr[p-b.ir] : value_type(0);
   }
 
   template <class PT1, class PT2, int shift>
@@ -722,6 +479,9 @@ namespace gmm {
   (std::ostream &o, const cs_vector_ref<PT1, PT2, shift>& m)
   { gmm::write(o,m); return o; }
 
+  template <class PT1, class PT2, int shift>
+  inline size_type nnz(const cs_vector_ref<PT1, PT2, shift>& l) { return l.n; }
+
   /* ******************************************************************** */
   /*	    Read only reference on a compressed sparse column matrix      */
   /* ******************************************************************** */
@@ -769,9 +529,9 @@ namespace gmm {
 
   template <class PT1, class PT2, class PT3, int shift = 0>
   struct csc_matrix_ref {
-    PT1 pr;
-    PT2 ir;
-    PT3 jc;
+    PT1 pr; // values.
+    PT2 ir; // row indexes.
+    PT3 jc; // column repartition on pr and ir.
     size_type nc, nr;
     
     typedef typename std::iterator_traits<PT1>::value_type value_type;
@@ -850,9 +610,9 @@ namespace gmm {
 
   template <class PT1, class PT2, class PT3, int shift = 0>
   struct csr_matrix_ref {
-    PT1 pr;
-    PT2 ir;
-    PT3 jc;
+    PT1 pr; // values.
+    PT2 ir; // column indexes.
+    PT3 jc; // row repartition on pr and ir.
     size_type nc, nr;
     
     typedef typename std::iterator_traits<PT1>::value_type value_type;
