@@ -1,5 +1,5 @@
 $er = 0;
-open F, "dynamic_array 2>&1 |" or die;
+open F, "./dynamic_array 2>&1 |" or die;
 while (<F>) {
   # print $_;
   if ($_ =~ /error has been detected/)
@@ -10,6 +10,6 @@ while (<F>) {
   }
 }
 if ($er == 1) { exit(1); }
-`dynamic_array`;
+`./dynamic_array`;
 if ($?) { exit(1); }
 
