@@ -27,7 +27,7 @@ close(TMPF);
 
 
 $er = 0;
-open F, "elasto_statique $tmp 2>&1 |" or die;
+open F, "./elasto_statique $tmp 2>&1 |" or die;
 while (<F>) {
   # print $_;
   if ($_ =~ /error has been detected/)
@@ -37,7 +37,8 @@ while (<F>) {
     print $_, <F>;
   }
 }
-if ($er == 1) { exit(1); }
-`elasto_statique $tmp`;
+if ($er == 1) { `rm -f $tmp`; exit(1); }
+`./elasto_statique $tmp`;
+`rm -f $tmp`;
 if ($?) { exit(1); }
 
