@@ -17,7 +17,7 @@ print TMPF "\n\n";
 close(TMPF);
 
 $er = 0;
-open F, "geo_trans_inv $tmp 2>&1 |" or die;
+open F, "./geo_trans_inv $tmp 2>&1 |" or die;
 while (<F>) {
   # print $_;
   if ($_ =~ /error has been detected/)
@@ -27,6 +27,7 @@ while (<F>) {
     print $_, <F>;
   }
 }
-if ($er == 1) { exit(1); }
-`geo_trans_inv $tmp`;
+if ($er == 1) { `rm -f $tmp`; exit(1); }
+`./geo_trans_inv $tmp`;
+`rm -f $tmp`; 
 if ($?) { exit(1); }
