@@ -19,6 +19,7 @@ print TMPF "NX = 7;\n";
 print TMPF "RESIDU = 1E-9;\n";
 print TMPF "FEM_TYPE = 0;\n"; 
 print TMPF "ROOTFILENAME = 'laplacian';\n";
+print TMPF "GENERIC_DIRICHLET = 1;\n";
 print TMPF "\n\n";
 close(TMPF);
 
@@ -75,6 +76,8 @@ print ".";
 start_program("-d INTEGRATION=2 -d MESH_TYPE=1 -d -d NX=10 INCLINE=0.5");
 print ".";
 start_program("-d N=1 -d FEM_TYPE=2 -d FT=1.0");
+print ".";
+start_program("-d K=2 -d KI=2 -d N=3 -d NX=1 -d FEM_TYPE=3 -d INTEGRATION=3 FT=1.0");
 print ".\n";
 
 `rm -f $tmp`;
